-- Imports custom CombatAlert modules
import "CombatAlert.Utils"
import "CombatAlert.Settings"

-- Defines the module locally
local AlertLogic = {}

-- Local reference to the main CombatAlert window
local CA

-- Handles combat status changes and triggers appropriate alerts
function AlertLogic.combatChanged(sender, args)
    if _G.CombatAlert.player then
        local inCombat = _G.CombatAlert.player:IsInCombat()
        local settings = _G.CombatAlert.Settings.get()

        if inCombat then
            AlertLogic.triggerAlert(
                3, true, settings.entercombatalert, settings.enterColor,
                settings.enterCombatText or "Entering Combat"
            )
        elseif settings.leavecombatalert then
            AlertLogic.triggerAlert(
                1.5, false, true, settings.leaveColor,
                settings.leaveCombatText or "Leaving Combat"
            )
        end
    end
end

-- Triggers an alert
function AlertLogic.triggerAlert(duration, isEnterCombat, textAlert, color,
    text)
    -- Does not trigger alerts during initialization
    if _G.CombatAlert.isInitializing then
        return
    end

    if CA then
        _G.CombatAlert.sleepEnd = Turbine.Engine.GetGameTime() + duration
        local settings = _G.CombatAlert.Settings.get()

        _G.CombatAlert.AlertUI.setupAlertDisplay(
            isEnterCombat, settings.visualalert
        )
        _G.CombatAlert.AlertUI.setupAlertLabel(
            textAlert, color, text, settings.textZoom or 100
        )

        _G.CombatAlert.AlertUI.setAlertVisibility(true, false)
        CA:SetWantsUpdates(true)
    end
end

-- Previews alerts
function AlertLogic.previewAlert(isEnterCombat, forcePreview)
    local settings = _G.CombatAlert.Settings.get()
    if isEnterCombat then
        AlertLogic.triggerAlert(
            3, true, settings.entercombatalert, settings.enterColor or
            {0, 1, 0, 1}, settings.enterCombatText or "Entering Combat"
        )
    elseif settings.leavecombatalert or forcePreview then
        AlertLogic.triggerAlert(
            1.5, false, true, settings.leaveColor or {0, 0, 210 / 255, 1},
            settings.leaveCombatText or "Leaving Combat"
        )
    end
end

-- Initializes the AlertLogic module
function AlertLogic.initialize(windowInstance)
    CA = windowInstance

    CA.Update = function()
        if Turbine.Engine.GetGameTime() > _G.CombatAlert.sleepEnd then
            _G.CombatAlert.AlertUI.setAlertVisibility(false, false)
            CA:SetVisible(false)
            CA:SetWantsUpdates(false)
        end
    end
end

-- Makes AlertLogic globally accessible
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.AlertLogic = AlertLogic

return AlertLogic
