-- Defines the Fonts module
local Fonts = {}

-- List of available fonts
Fonts.availableFonts = {
    "Default", "Arial", "Book Antiqua", "Book Antiqua Bold", "FixedSys",
    "Lucida Console", "Trajan Pro", "Verdana", "Verdana Bold"
}

-- Mapping of font names to their corresponding Turbine.UI.Lotro.Font variables
-- and zoom modifiers as a rough attempt to try and normalize font sizes
Fonts.fontMapping = {
    ["Default"] = {
        font = Turbine.UI.Lotro.Font.TrajanProBold36,
        modifier = 1
    },
    ["Arial"] = {
        font = Turbine.UI.Lotro.Font.Arial12,
        modifier = 4
    },
    ["Book Antiqua"] = {
        font = Turbine.UI.Lotro.Font.BookAntiqua24,
        modifier = 2
    },
    ["Book Antiqua Bold"] = {
        font = Turbine.UI.Lotro.Font.BookAntiquaBold24,
        modifier = 2
    },
    ["FixedSys"] = {
        font = Turbine.UI.Lotro.Font.FixedSys15,
        modifier = 2.5
    },
    ["Lucida Console"] = {
        font = Turbine.UI.Lotro.Font.LucidaConsole12,
        modifier = 3
    },
    ["Trajan Pro"] = {
        font = Turbine.UI.Lotro.Font.TrajanPro28,
        modifier = 1.5
    },
    ["Verdana"] = {
        font = Turbine.UI.Lotro.Font.Verdana23,
        modifier = 2
    },
    ["Verdana Bold"] = {
        font = Turbine.UI.Lotro.Font.VerdanaBold16,
        modifier = 2.5
    }
}

-- Creates the font dropdown
function Fonts.createFontDropdown(parent, position)
    local dropdownWidth = 200
    local labelWidth = 100
    local fontLabel = _G.CombatAlert.Options.Controls.createLabel(
        parent,
        {labelWidth, 20},
        {position.x, position.y},
        "Alert Font:"
    )

    local dropdown = Turbine.UI.Control()
    dropdown:SetParent(parent)
    dropdown:SetSize(dropdownWidth, 20)
    dropdown:SetPosition(position.x + labelWidth + 36, position.y)
    dropdown:SetBackColor(Turbine.UI.Color(0.2, 0.2, 0.2))

    local selectedValueLabel = Turbine.UI.Label()
    selectedValueLabel:SetParent(dropdown)
    selectedValueLabel:SetSize(dropdownWidth - 20, 20)
    selectedValueLabel:SetPosition(5, 0)
    selectedValueLabel:SetText("Default")
    selectedValueLabel:SetFont(Turbine.UI.Lotro.Font.TrajanPro15)
    selectedValueLabel:SetForeColor(Turbine.UI.Color(0.8, 0.75, 0.5))
    selectedValueLabel:SetTextAlignment(
        Turbine.UI.ContentAlignment.MiddleLeft
    )

    local dropDownArrow = Turbine.UI.Control()
    dropDownArrow:SetParent(dropdown)
    dropDownArrow:SetSize(16, 16)
    dropDownArrow:SetPosition(dropdownWidth - 16, 2)
    dropDownArrow:SetBackground("CombatAlert/Resources/DropdownArrow.tga")

    local listBox = Turbine.UI.ListBox()
    listBox:SetParent(parent)
    listBox:SetPosition(
        dropdown:GetLeft(),
        dropdown:GetTop() + dropdown:GetHeight()
    )
    listBox:SetSize(dropdownWidth, 180)
    listBox:SetBackColor(Turbine.UI.Color(0.1, 0.1, 0.1))
    listBox:SetVisible(false)
    listBox:SetZOrder(9999)

    for _, fontName in ipairs(Fonts.availableFonts) do
        local item = Turbine.UI.Label()
        item:SetSize(dropdownWidth, 20)
        item:SetText(fontName)
        item:SetFont(Turbine.UI.Lotro.Font.TrajanPro15)
        item:SetForeColor(Turbine.UI.Color(0.8, 0.75, 0.5))
        item:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
        item:SetMouseVisible(true)
        item.MouseClick = function(sender, args)
            selectedValueLabel:SetText(fontName)
            listBox:SetVisible(false)
            Fonts.setFont(fontName)
            _G.CombatAlert.AlertLogic.previewAlert(false)
        end
        listBox:AddItem(item)
    end

    dropdown.MouseClick = function(sender, args)
        listBox:SetVisible(not listBox:IsVisible())
    end

    dropDownArrow.MouseClick = dropdown.MouseClick

    return dropdown, dropDownArrow, fontLabel, selectedValueLabel, listBox
end

-- Sets the font for the alert labels
function Fonts.setFont(fontName)
    _G.CombatAlert.Settings.set("alertFont", fontName)
    if _G.CombatAlert.AlertUI and _G.CombatAlert.AlertUI.updateTextZoom then
        local currentZoom = _G.CombatAlert.Settings.get("textZoom") or 100
        _G.CombatAlert.AlertUI.updateTextZoom(currentZoom)
    end
end

-- Gets the zoom modifier for a given font
function Fonts.getZoomModifier(fontName)
    local fontData = Fonts.fontMapping[fontName]
    return fontData and fontData.modifier or 1
end

-- Makes Fonts globally accessible
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.Options = _G.CombatAlert.Options or {}
_G.CombatAlert.Options.Fonts = Fonts

return Fonts
