-- Defines the Tab class as a subclass of Turbine.UI.Control
local Tab = class(Turbine.UI.Control)

-- Constructs a new Tab
function Tab:Constructor(parent, tabIndex, tabName, tabContent)
    Turbine.UI.Control.Constructor(self)

    self.enabled = tabContent ~= nil
    self.index = tabIndex
    self.parent = parent
    self.content = tabContent
    self.active = false

    if self.enabled then
        self.content:SetPosition(2, 24)
    end

    self:SetMouseVisible(true)
    self:SetParent(self.parent)
    self:SetPosition((self.index - 1) * 98, 2)
    self:SetSize(98, 21)

    -- Creates background for the tab
    self.background = Turbine.UI.Control()
    self.background:SetParent(self)
    self.background:SetSize(98, 21)
    self.background:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.background:SetMouseVisible(false)
    self.background:SetBackground(
        "CombatAlert/Resources/TabInactive.tga")

    -- Creates text label for the tab
    self.text = Turbine.UI.Label()
    self.text:SetParent(self)
    self.text:SetText(tabName)
    self.text:SetTop(1)
    self.text:SetSize(98, 20)
    self.text:SetFont(Turbine.UI.Lotro.Font.TrajanPro15)
    self.text:SetForeColor(Turbine.UI.Color(212/255, 175/255, 55/255))
    self.text:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.text:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    self.text:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.text:SetMouseVisible(false)
end

-- Toggles the active state of the tab
function Tab:toggleActive(active)
    if active == self.active then return end

    self.active = active

    if self.active then
        self.text:SetForeColor(Turbine.UI.Color(1, 1, 1))
        self.text:SetFont(Turbine.UI.Lotro.Font.TrajanPro16)
        self.text:SetText(self.text:GetText())
        self.background:SetBackground(
            "CombatAlert/Resources/TabActive.tga")
        self.content:SetVisible(true)
        self.content:SetParent(self.parent.tabContent)
    else
        self.text:SetForeColor(Turbine.UI.Color(212/255, 175/255, 55/255))
        self.text:SetFont(Turbine.UI.Lotro.Font.TrajanPro15)
        self.text:SetText(self.text:GetText())
        self.background:SetBackground(
            "CombatAlert/Resources/TabInactive.tga")
        self.content:SetVisible(false)
        self.content:SetParent(nil)
    end
end

-- Handles mouse enter event
function Tab:MouseEnter(args)
    if not self.active then
        self.text:SetForeColor(Turbine.UI.Color(1, 1, 1))
        self.text:SetFont(Turbine.UI.Lotro.Font.TrajanPro16)
        self.text:SetText(self.text:GetText())
        self.background:SetBackground(
            "CombatAlert/Resources/TabActive.tga")
    end
end

-- Handles mouse leave event
function Tab:MouseLeave(args)
    if not self.active then
        self.text:SetForeColor(Turbine.UI.Color(212/255, 175/255, 55/255))
        self.text:SetFont(Turbine.UI.Lotro.Font.TrajanPro15)
        self.text:SetText(self.text:GetText())
        self.background:SetBackground(
            "CombatAlert/Resources/TabInactive.tga")
    end
end

-- Handles mouse click event
function Tab:MouseClick(args)
    self.parent:selectTab(self.index)
end

-- Defines the TabManager class
local TabManager = class(Turbine.UI.Control)

-- Constructs a new TabManager
function TabManager:Constructor()
    Turbine.UI.Control.Constructor(self)
    self.tabs = {}
    self.activeTab = 0
    self.tabCount = 0

    -- Creates content area for tabs
    self.tabContent = Turbine.UI.Control()
    self.tabContent:SetParent(self)
    self.tabContent:SetPosition(0, 23)
    self.tabContent:SetSize(518, 467)
    self.tabContent:SetBackColor(Turbine.UI.Color(0.05, 0.05, 0.1))
end

-- Adds a new tab to the manager
function TabManager:addTab(tabName, tabContent)
    self.tabCount = self.tabCount + 1
    local newTab = Tab(self, self.tabCount, tabName, tabContent)
    newTab:SetZOrder(2)
    table.insert(self.tabs, newTab)
    if self.tabCount == 1 then
        self:selectTab(1)
    end
end

-- Selects a tab by index
function TabManager:selectTab(tabIndex)
    if tabIndex == self.activeTab or tabIndex > self.tabCount then return end
    if self.activeTab ~= 0 then
        self.tabs[self.activeTab]:toggleActive(false)
    end
    self.activeTab = tabIndex
    local tab = self.tabs[self.activeTab]
    if tab ~= nil then
        tab:toggleActive(true)
        tab.content:SetParent(self.tabContent)
        tab.content:SetPosition(0, 0)
        tab.content:SetSize(self.tabContent:GetSize())
    end
    if self.OnTabSelected then
        self.OnTabSelected(tabIndex)
    end
end

-- Sets the size of the TabManager
function TabManager:SetSize(width, height)
    Turbine.UI.Control.SetSize(self, width, height)
    if self.tabContent then
        self.tabContent:SetSize(width, height - 23)
    end
end

-- Makes TabManager globally accessible
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.Options = _G.CombatAlert.Options or {}
_G.CombatAlert.Options.Tabs = TabManager

return TabManager
