-- Imports custom CombatAlert modules
import "CombatAlert.Utils"

-- Defines the module locally
local Settings = {}

-- Defines default settings
local defaults = {
    visualalert = true,
    entercombatalert = true,
    leavecombatalert = true,
    alertColor = "Default",
    alertImageFile = "CombatAlert/Resources/AlertImage-Default.tga",
    enterCombatText = "Entering Combat",
    leaveCombatText = "Leaving Combat",
    enterColor = {1, 1, 0, 0},  -- A, R, G, B: Red
    leaveColor = {1, 0, 0.824, 0},  -- A, R, G, B: Green
    textZoom = 100,
    opacity = 255,
    alertColorPresets = {
        Default = {1, 0.796, 0.298, 0.251},
        Custom = {1, 0.796, 0.298, 0.251},
        Blue = {1, 0, 0, 1},
        Cyan = {1, 0, 1, 1},
        Pink = {1, 1, 0.75, 0.8},
        Purple = {1, 0.5, 0, 0.5},
        Red = {1, 1, 0, 0},
        Green = {1, 0, 1, 0},
        Silver = {1, 0.75, 0.75, 0.75},
        Gold = {1, 1, 0.84, 0},
        Orange = {1, 1, 0.65, 0}
    }
}

-- Defines temporary settings storage
local currentSettings = {}

-- Saves current settings to plugin data
local function save()
    Turbine.PluginData.Save(Turbine.DataScope.Account, "CombatAlert",
                            currentSettings)
end

-- Loads settings from plugin data, applying defaults where necessary
function Settings.load()
    local loadedSettings = Turbine.PluginData.Load(Turbine.DataScope.Account,
                                                   "CombatAlert") or {}

    for key, value in pairs(defaults) do
        if loadedSettings[key] == nil then
            loadedSettings[key] = value
        elseif key == "textZoom" or key == "opacity" then
            loadedSettings[key] = tonumber(loadedSettings[key]) or value
        elseif key == "alertColorPresets" then
            loadedSettings[key].Custom = loadedSettings[key].Custom or
                                         value.Custom
        end
    end

    currentSettings = loadedSettings
    return loadedSettings
end

-- Retrieves a specific setting or all settings if no key is provided
function Settings.get(key)
    if key == "textZoom" or key == "opacity" then
        return tonumber(currentSettings[key]) or defaults[key]
    end
    return key and currentSettings[key] or currentSettings
end

-- Sets a specific setting and saves the changes
function Settings.set(key, value)
    if key and value ~= nil then
        if key == "textZoom" or key == "opacity" then
            value = tonumber(value) or defaults[key]
        end
        currentSettings[key] = value
        save()
    end
end

-- Resets all settings to their default values
function Settings.resetToDefaults()
    currentSettings = _G.CombatAlert.Utils.deepCopy(defaults)
    save()
    return currentSettings
end

-- Makes Settings globally accessible
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.Settings = Settings

Settings.defaults = defaults

return Settings
